// testlbl.cpp - Tests RtPlot by adding line-by-line
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <windows.h>
#include "mutex.hpp"

bool set_current_file_time( const char *fname );
void print_plot(const char *fname, double xstart, double dx,
		long int sleep, const double XMAX, Mutex& mutex );
//Set new file time
bool 
set_current_file_time( const char *fname )
{
  HANDLE h = 
    ::CreateFile( fname, GENERIC_WRITE, FILE_SHARE_WRITE,
		  NULL, OPEN_EXISTING, 0, NULL );
  bool ret_val = true;
  if( INVALID_HANDLE_VALUE == h ) {
    DWORD errcode = ::GetLastError();
    fprintf( stderr, "Create file failed with code %ld ", errcode );
    fprintf( stderr, "\n");
    fflush( stderr );
    return false;
    //exit(EXIT_FAILURE);
  } else {
    FILETIME write_time;
    ::GetSystemTimeAsFileTime(&write_time);
    BOOL is_ok=::SetFileTime(h, NULL, NULL, &write_time);
    if(!is_ok) {
      DWORD errcode = ::GetLastError();
      fprintf(stderr, "Error setting file time code %ld ", errcode);
      fprintf( stderr, "\n");
      fflush( stderr );
      return false;
    }    
    CloseHandle(h);
  }
  return ret_val;
}

void
print_plot(const char *fname, double xstart, double dx,
	   long int sleep, const double XMAX, Mutex& mutex  )
{
  double x = -dx;
  FILE *f = fopen(fname, "w");
  fprintf(f,"%s \n", "#Output from plottest program");
  fflush(f);
  bool b;
  while(1) {
    x += dx;
    if( x > XMAX ) break;
    double ysin = sin(x);
    double ycos = cos(x);
    {
      Lock lock(mutex);
      DWORD mutex_ret_val = mutex.wait_return_code();
      switch( mutex_ret_val ) {
      case WAIT_OBJECT_0:
//	fprintf(f, "%8.3f %8.3f %8.3f", x, ysin, ycos);
	fprintf(f, "%.3f,%.3f,%.3f", x, ysin, ycos);
	fprintf(f, "\n");
	fflush(f);
	b = set_current_file_time( fname );
	if( !b ) {
	  exit(EXIT_FAILURE);
	}
	break;
      case WAIT_ABANDONED:
	MessageBox(NULL, "Mutex returned with ABANDONED",  "testplot:", MB_OK );
        break;
      case WAIT_TIMEOUT:
	MessageBox(NULL, "Mutex retruned with TIMEOUT", "testplot:", MB_OK );
	break;
      default:
	break;
      }
    }
    ::Sleep(sleep);
  }
  fclose(f);
  return;
}

int 
main(int argc, char **argv)
{
  const double XMAX = 20.0;
  const double dx = (2*acos(-1))/64;
  Mutex mutex( "RtPlotMutex", INFINITE);
  
  while(1) {
    print_plot("testplot.txt", 0.0, dx, 1000L, XMAX, mutex ); 
  }
  return 0;
}
